from .base_extension import BaseExtension, ExtensionInput
from .extension_host import ExtensionHost
from .extension_metadata import ExtensionMetadata

__all__ = ["BaseExtension", "ExtensionInput", "ExtensionHost", "ExtensionMetadata"]



from typing import Optional
import httpx
from langchain_openai_api_bridge.core.base_agent_factory import BaseAgentFactory
from .base_extension import BaseExtension
from langchain_core.runnables import Runnable
from langchain_openai import ChatOpenAI
from langchain_openai_api_bridge.core.create_agent_dto import CreateAgentDto


class ExtensionAgentFactory(BaseAgentFactory):

    def __init__(
        self,
        extension: BaseExtension,
        inference_url: Optional[str],
        http_client: httpx.Client | None = None,
    ) -> None:
        self.extension = extension
        self.inference_url = inference_url
        self.http_client = http_client

    def create_agent(self, dto: CreateAgentDto) -> Runnable:
        return self.extension.create_runnable(
            llm=ChatOpenAI(
                base_url=self.inference_url,
                http_client=self.http_client,
                temperature=dto.temperature or 0.7,
                model=dto.model,
                max_tokens=dto.max_tokens,
                api_key=dto.api_key,
            )
        )



from abc import ABC, abstractmethod
from typing import TypedDict
from langchain_core.runnables import Runnable
from langchain_core.language_models import BaseChatModel
from langchain_core.messages import BaseMessage


class ExtensionInput(TypedDict):
    messages: list[BaseMessage]


class BaseExtension(ABC):
    @abstractmethod
    def name(self) -> str:
        pass

    @abstractmethod
    def description(self) -> str:
        """
        Will be used by LLM for tool calling selection.
        It's prefered to have a relevant description
        """
        pass

    def setup(self) -> None:
        """
        Called when setuping the extension
        """
        pass

    @abstractmethod
    def create_runnable(self, llm: BaseChatModel) -> Runnable:
        pass



from typing import Optional
import httpx
import uvicorn
from .extension_agent_factory import ExtensionAgentFactory
from .extension_metadata import ExtensionMetadata
from .base_extension import BaseExtension
from langchain_openai_api_bridge.fastapi.langchain_openai_api_bridge_fastapi import (
    LangchainOpenaiApiBridgeFastAPI,
)
from fastapi import FastAPI


class ExtensionHost:
    def __init__(
        self,
        extension: BaseExtension,
        port: Optional[int] = 7680,
        inference_url: Optional[str] = None,
        inference_http_client: httpx.Client | None = None,
    ) -> None:
        self.extension = extension
        self.port = port

        self.app = FastAPI()
        self.bridge = LangchainOpenaiApiBridgeFastAPI(
            app=self.app,
            agent_factory_provider=ExtensionAgentFactory(
                extension=self.extension,
                inference_url=inference_url,
                http_client=inference_http_client,
            ),
        )

        self._setup_routes()
        self.setup()

    def _setup_routes(self):
        @self.app.get("/metadata")
        def get_metadata():
            return ExtensionMetadata(
                name=self.extension.name(),
                description=self.extension.description(),
            )

        self.bridge.bind_openai_chat_completion()

    def setup(self):
        if hasattr(self.extension, "setup"):
            self.extension.setup()

    def start_server(self):
        uvicorn.run(self.app, host="localhost", port=self.port)
